module DB where

import Data.List.Split
import Control.Monad
import Database.HDBC
import Database.HDBC.Sqlite3

data Link = Link
  {
    linkID :: Int,
    outgoingLinksCount :: Int,
    incomingLinksCount :: Int,
    outgoingLinks :: [String],
    incomingLinks :: [String]
  } deriving Show

data Page = Page
  {
    pageID :: Int,
    pageTitle :: String,
    pageIsRedirect :: Bool
  } deriving Show

data Redirect = Redirect
  {
    redirectSourceID :: Int,
    redirectTargetID :: Int
  } deriving Show


rowToPage :: [SqlValue] -> Page
rowToPage (id:title:isRedirect:_) = Page (fromSql id) (fromSql title) $ ((fromSql isRedirect)::Integer) == 1

getPageFromTitle :: IO Connection -> String -> IO Page
getPageFromTitle conn title = do
  c <- conn
  rows <- quickQuery c "SELECT * FROM pages WHERE title=?" [toSql title]
  let pages = map rowToPage rows
  return (head pages)

getPageFromID :: IO Connection -> Int -> IO Page
getPageFromID conn id = do
  c <- conn
  rows <- quickQuery c "SELECT * FROM pages WHERE id=?" [toSql id]
  let pages = map rowToPage rows
  return (head pages)

getOutgoingLinksForPages :: IO Connection -> [Int] -> IO [[Int]]
getOutgoingLinksForPages conn pageIDs = do
  c <- conn
  rows <- quickQuery c (buildStmt "SELECT id, outgoing_links FROM links WHERE id IN (" (length pageIDs)) (map toSql pageIDs)
  return $ map getLink rows
    where
      getLink (_:outgoing:_) = map (\x -> (read x::Int)) $ splitOn "|" $ fromSql outgoing

getOutgoingLinksForPage :: IO Connection -> Int -> IO [Int]
getOutgoingLinksForPage conn pageID = do
  results <- getOutgoingLinksForPages conn [pageID]
  return $ head results

getIncomingLinksForPages :: IO Connection -> [Int] -> IO [[Int]]
getIncomingLinksForPages conn pageIDs = do
  c <- conn
  rows <- quickQuery c (buildStmt "SELECT id, incoming_links FROM links WHERE id IN (" (length pageIDs)) (map toSql pageIDs)
  return $ map getLink rows
    where
      getLink (_:incoming:_) = map (\x -> (read x::Int)) $ splitOn "|" $ fromSql incoming


getIncomingLinksCounts :: IO Connection -> [Int] -> IO Int
getIncomingLinksCounts conn pageIDs = do
  c <- conn
  rows <- quickQuery c (buildStmt "SELECT SUM(incoming_links_count) FROM links WHERE id IN (" (length pageIDs)) (map toSql pageIDs)
  let value = head $ head rows
  return $ fromSql value

getOutgoingLinksCounts :: IO Connection -> [Int] -> IO Int
getOutgoingLinksCounts conn pageIDs = do
  c <- conn
  rows <- quickQuery c (buildStmt "SELECT SUM(outgoing_links_count) FROM links WHERE id IN (" (length pageIDs)) (map toSql pageIDs)
  let value = head $ head rows
  return $ fromSql value

buildStmt :: String -> Int -> String
buildStmt stmt 1 = stmt ++ "?);"
buildStmt stmt n = buildStmt (stmt ++ "?, ") (n - 1)
